//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include "azac_api_cxx_details_lifetime.hpp"
#include "vision_api_cxx_common.hpp"
#include "vision_api_cxx_image_writer.hpp"

#include <vision_api_c_frame_source.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Input {

/// <summary>
/// Represents a source of image data, used as input to or output from Vision AI operations.
/// </summary>
class ImageSourceBuffer
{
protected:
    /*! \cond PROTECTED */
    template<typename Target> using ProtectedAccess = ::Azure::AI::Core::_detail::ProtectedAccess<Target>;
    /*! \endcond */

public:

    using CallbackFunction_Type = ::std::function<void(const std::shared_ptr<ImageWriter>& writer)>;

    /// <summary>
    /// Constructor that takes an optional callback function to write a new image
    /// </summary>
    /// <param name="callback">A callback function that will be invoked when a new image is needed</param>
    /// <remarks>
    /// Supported for usage with ImageAnalyzer, but without a callback. Call this constructor without arguments.
    /// Then call GetWriter() on the resulting object to get an ImageWriter.
    /// </remarks>
    ImageSourceBuffer(const CallbackFunction_Type callback = nullptr) :
        m_frameSource(AZAC_HANDLE_INVALID),
        m_binder(AZAC_HANDLE_INVALID, ReleaseCallback),
        m_callback(callback)
    {

        AZAC_HANDLE handle = AZAC_HANDLE_INVALID;
        AZAC_THROW_ON_FAIL(::vision_frame_source_handle_create(&handle, "adapter.streams.count", "1", nullptr, "adapter.streams.0."));

        Azure::AI::Core::_detail::LifetimeBinder binder(handle, ReleaseCallback);

        m_frameSource = handle;
        m_binder = binder;
    }

    /// <summary>
    /// Destructs an instance of the ImageSourceBuffer class.
    /// </summary>
    virtual ~ImageSourceBuffer()
    {
        m_frameSource = AZAC_HANDLE_INVALID;
    };

    /// <summary>
    /// Gets the ImageWriter associated with this ImageSourceBuffer
    /// </summary>
    /// <returns>An ImageWriter as std::shared_ptr</returns>
    /// <remarks>
    /// Use the ImageWriter's Write method to write an image to the internal buffer.
    /// </remarks>
    std::shared_ptr<ImageWriter> GetWriter()
    {
        AZAC_HANDLE handle = AZAC_HANDLE_INVALID;
        AZAC_THROW_ON_FAIL(vision_frame_source_writer_handle_get(m_frameSource, &handle));
        return ProtectedAccess<ImageWriter>::FromHandle(handle, m_binder);
    }

    /// <summary>
    /// Closes the ImageSourceBuffer for writing
    /// </summary>
    void CloseWriter()
    {
        AZAC_THROW_ON_FAIL(vision_frame_source_close_writer(m_frameSource));
    }

private:
    friend class Azure::AI::Core::_detail::ProtectedAccess<ImageSourceBuffer>;

    explicit operator AZAC_HANDLE() { return m_frameSource; }

    AZAC_HANDLE m_frameSource;
    Azure::AI::Core::_detail::LifetimeBinder m_binder;
    CallbackFunction_Type m_callback;

    static void ReleaseCallback(void * context)
    {
        auto sourceHandle = reinterpret_cast<AZAC_HANDLE>(context);
        if (vision_frame_source_handle_is_valid(sourceHandle))
        {
            ::vision_frame_source_handle_release(sourceHandle);
        }
    }
};

} } } } // Azure::AI::Vision::Input
