//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once

#include "azac_api_cxx_details_lifetime.hpp"
#include "vision_api_cxx_common.hpp"

#include <vision_api_c_frame_writer.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Input {

/// <summary>
/// Represents the ability to write image data, for use as input to Vision AI operations.
/// </summary>
class ImageWriter
{
public:

    /// <summary>
    /// Destructs an instance of the ImageWriter class.
    /// </summary>
    virtual ~ImageWriter()
    {
        if (vision_frame_writer_handle_is_valid(m_frameWriter))
        {
            vision_frame_writer_handle_release(m_frameWriter);
            m_frameWriter = AZAC_HANDLE_INVALID;
        }
    }

    /// <summary>
    /// Writes a single image to the internal buffer.
    /// </summary>
    /// <param name="data">The image data buffer</param>
    /// <param name="dataSizeInBytes">The number of bytes to copy from the data buffer</param>
    /// <remarks>
    /// When used with ImageAnalyzer, the image needs to be in a format that's supported by the Image Analysis service, such as JPEG, PNG or BMP.
    /// See full list of supported formats here: https://aka.ms/ia-input.
    /// The SDK sends the image buffer as-is to the service. It does not do any format conversion or other modifications.
    /// </remarks>
    void Write(const uint8_t* data, size_t dataSizeInBytes)
    {
        if (data == nullptr || dataSizeInBytes == 0)
        {
            throw std::invalid_argument("Invalid input buffer");
        }
        uint32_t size = (uint32_t)dataSizeInBytes; // idiomatic to accept size_t, but C API uses uint32_t
        AZAC_THROW_ON_FAIL(vision_frame_writer_write(m_frameWriter, 0, data, size, nullptr, nullptr));
    }

private:

    friend class Azure::AI::Core::_detail::ProtectedAccess<ImageWriter>;

    static std::shared_ptr<ImageWriter> FromHandle(AZAC_HANDLE handle, const Azure::AI::Core::_detail::LifetimeBinder& binder)
    {
        auto ptr = new ImageWriter(handle, binder);
        return std::shared_ptr<ImageWriter>(ptr);
    }

    explicit ImageWriter(AZAC_HANDLE frameWriter, const Azure::AI::Core::_detail::LifetimeBinder& binder) :
        m_frameWriter(frameWriter),
        m_binder{ binder }
    {
    }

    explicit operator AZAC_HANDLE() { return m_frameWriter; }
 
    AZAC_DISABLE_DEFAULT_CTORS(ImageWriter);

    AZAC_HANDLE m_frameWriter;
    Azure::AI::Core::_detail::LifetimeBinder m_binder;
};

} } } } // Azure::AI::Vision::Input
